#include "opcodes.h"

// clang-format off
opcode_data_t *const opcodes[] = {
    // RET
    &(opcode_data_t) {
        .mnemonic = "ret",
        .opcode = 0xC3,
        .opcode_extension = opcode_extension_none,
        .operand_count = 0,
    },
    // RET imm16
    &(opcode_data_t) {
        .mnemonic = "ret",
        .opcode = 0xC2,
        .opcode_extension = opcode_extension_none,
        .operand_count = 1,
        .operands = {
            { .kind = OPERAND_IMMEDIATE, .size = OPERAND_SIZE_16 },
        },
    },
    // PUSH imm8
    &(opcode_data_t) {
        .mnemonic = "push",
        .opcode = 0x6A,
        .opcode_extension = opcode_extension_none,
        .operand_count = 1,
        .operands = {
            { .kind = OPERAND_IMMEDIATE, .size = OPERAND_SIZE_8},
        },
    },
    // PUSH imm16
    &(opcode_data_t) {
        .mnemonic = "push",
        .opcode = 0x68,
        .opcode_extension = opcode_extension_none,
        .operand_size_prefix = true,
        .operand_count = 1,
        .operands = {
            { .kind = OPERAND_IMMEDIATE, .size = OPERAND_SIZE_16},
        },
    },
    // PUSH imm32
    &(opcode_data_t) {
        .mnemonic = "push",
        .opcode = 0x68,
        .opcode_extension = opcode_extension_none,
        .operand_size_prefix = false,
        .operand_count = 1,
        .operands = {
            { .kind = OPERAND_IMMEDIATE, .size = OPERAND_SIZE_32},
        },
    },
    // Push reg16, 
    &(opcode_data_t) {
        .mnemonic = "push",
        .opcode = 0x50,
        .opcode_extension = opcode_extension_none,
        .encoding_class = ENCODING_OPCODE_REGISTER,
        .operand_count = 1,
        .operands = {
            { .kind = OPERAND_REGISTER, .size = OPERAND_SIZE_16 | OPERAND_SIZE_32 | OPERAND_SIZE_64 },
        },
    },

    nullptr,
};

